/**
 * This class contains functions specific to the Inventory tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function InventoryTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "inventorytab";
    var title = "Inventory";
    var sellButtonClass = "itemrowsell";
    var useButtonClass = "itemrowuse";
    var coinSpanId = "coins";
    var blinking = false;
    var blinkClass = "blinktab";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds and returns a HTML-subtitle.
     * @param {string} subtitle The actual subtitle.
     * @returns {string} The subtitle as a HTML-subtitle.
     */
    function buildSubtitle(subtitle) {
        return `<h2 class="subtitle">${subtitle}</h2>`;
    }

    /**
     * Builds a bold-faced item name which will reveal a tooltip when moused over.
     * @param {object} item The item for which to build a tooltip.
     * @param {boolean} caps Whether or not the item's name should be capitalized. May be omitted.
     * @returns {String} The constructed item name with tooltip.
     */
    function buildItemWithTooltip(item, caps) {
        return global.kobold.inventory.createItemWithTooltip(item, caps);
    }

    /**
     * Builds and returns the bra section of the equipment part of the inventory.
     * @returns {string} The bra section of the equipment part of the inventory as a HTML string.
     */
    function buildBra() {
        var isWearingBra = global.kobold.inventory.equipment.isWearing(SlotEnum.BRA);
        var genderHasBreasts = global.kobold.body.hasBreasts();
        // First authority = bra, second authority = gender.
        var shouldBeWearingBra = isWearingBra ? !global.kobold.inventory.equipment.bra.wrongGender() : genderHasBreasts;
        var bra = `<div class="text nomargin">`;

        if (isWearingBra && shouldBeWearingBra) {
            bra += `You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.bra)}.`;
            if (global.kobold.inventory.equipment.bra.skimpy()) {
                bra += ` <span class="warningtext">It barely covers your breasts.</span>`;
            } else {
                bra += ` It covers your chest nicely.`;
            }
        }

        if (isWearingBra && !shouldBeWearingBra) {
            bra += `You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.bra)}. <span class="sneakytext">Why are you wearing this?`;
            if (global.kobold.inventory.equipment.bra.skimpy()) {
                bra += " You look like a slut.";
            } else {
                bra += " You look silly.";
            }
            bra += "</span>";
        }

        if (!isWearingBra && shouldBeWearingBra) {
            bra += `You aren't wearing a bra.`;
            if (global.kobold.inventory.equipment.chestExposed()) {
                bra += ` <span class="warningtext">Your chest is completely exposed.</span>`;
            }
        }

        if (!isWearingBra && !shouldBeWearingBra) {
            bra += `You aren't wearing a bra. <span class="sneakytext">Why would you?</span>`;
        }

        return bra + "</div>";
    }

    /**
     * Builds and returns the helmet section of the equipment part of the inventory.
     * @returns {string} The helmet section of the equipment part of the inventory as a HTML string.
     */
    function buildHelmet() {
        var isWearingHelmet = global.kobold.inventory.equipment.isWearing(SlotEnum.HELMET);

        if (!isWearingHelmet) {
            return `<div class="text nomargin">You aren't wearing a helmet.</div>`;
        }

        var helmetConceals = global.kobold.inventory.equipment.helmet.hidesFace();
        var helmet = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.helmet)}.`;
        if (helmetConceals) {
            helmet += ` <span class="sneakytext">It conceals your face, allowing you to blend in more easily.</span>`;
        }
        return helmet + "</div>";
    }

    /**
     * Builds and returns the pants section of the equipment part of the inventory.
     * @returns {string} The pants section of the equipment part of the inventory as a HTML string.
     */
    function buildPants() {
        var isWearingPants = global.kobold.inventory.equipment.isWearing(SlotEnum.PANTS);
        var humiliate = global.kobold.body.hasCunt();

        if (isWearingPants) {
            return `<div class="text nomargin">You are wearing ${buildItemWithTooltip(global.kobold.inventory.equipment.pants)}.</div>`;
        }

        var pants = `<div class="text nomargin">You aren't wearing any pants.`;
        if (humiliate) {
            pants += ` <span class="warningtext">You slut.</span>`;
        }
        return pants + "</div>";
    }

    /**
     * Builds and returns the shirt section of the equipment part of the inventory.
     * @returns {string} The shirt section of the equipment part of the inventory as a HTML string.
     */
    function buildShirt() {
        var isWearingShirt = global.kobold.inventory.equipment.isWearing(SlotEnum.SHIRT);
        var coversGroin = global.kobold.inventory.equipment.shirt && global.kobold.inventory.equipment.shirt.coversGroin();
        var coversRear = global.kobold.inventory.equipment.shirt && global.kobold.inventory.equipment.shirt.coversRear();

        if (isWearingShirt) {
            var toReturn = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.shirt)}.`;
            if (coversGroin && coversRear) {
                toReturn += ` It's long enough to cover your waist.`;
            } else if (coversGroin) {
                toReturn += ` It's long enough to cover your groin.`;
            } else if (coversRear) {
                toReturn += ` It's long enough to cover your ass.`;
            }
            toReturn += `</div>`;
            return toReturn;
        }

        return `<div class="text nomargin">You aren't wearing a shirt.</div>`;
    }

    /**
     * Builds and returns the shoes section of the equipment part of the inventory.
     * @returns {string} The shoes section of the equipment part of the inventory as a HTML string.
     */
    function buildShoes() {
        var isWearingShoes = global.kobold.inventory.equipment.isWearing(SlotEnum.SHOES);

        if (!isWearingShoes) {
            return `<div class="text nomargin">You aren't wearing shoes. You're used to walking on your bare feet.</div>`;
        }

        var soleExposed = global.kobold.inventory.equipment.shoes.soleExposed();
        var shoe = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.shoes)}.`;
        if (soleExposed) {
            shoe += ` They do little to nothing to protect your soles.`;
        } else {
            shoe += ` Not walking on your bare feet for once is rather enjoyable.`;
        }
        return shoe + "</div>";
    }

    /**
     * Builds and returns the underwear section of the equipment part of the inventory.
     * @returns {string} The underwear section of the equipment part of the inventory as a HTML string.
     */
    function buildUnderwear() {
        var isWearingUnderwear = global.kobold.inventory.equipment.isWearing(SlotEnum.UNDERWEAR);
        var groinExposed = global.kobold.inventory.equipment.groinExposed();
        var rearExposed = global.kobold.inventory.equipment.rearExposed();

        var pantsu = "";
        if (!isWearingUnderwear) {
            pantsu += `<div class="text nomargin">You aren't wearing any underwear.`;
            if (groinExposed) {
                pantsu += ` <span class="warningtext">Your groin is completely naked.</span>`;
            }
            if (rearExposed) {
                pantsu += ` <span class="warningtext">Your butt is exposed.</span>`;
            }
            if (!groinExposed && !rearExposed) {
                pantsu += ` <span class="sneakytext">Luckily, another piece of clothing is covering your waist, hiding your indecency.</span>`;
            }
        }
        if (isWearingUnderwear) {
            pantsu += `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.underwear)}.`;
            if (groinExposed) {
                pantsu += ` <span class="warningtext">Your groin is still completely visible.</span>`;
            }
            if (global.kobold.inventory.equipment.underwear.coversGroin()) {
                pantsu += ` It hides your groin from prying eyes.`;
            }
            if (rearExposed) {
                pantsu += ` <span class="warningtext">Your rear is left uncovered.</span>`;
            }
            if (groinExposed && global.kobold.inventory.equipment.underwear.coversRear()) {
                pantsu += ` <span class="sneakytext">At least it covers your ass.</span>`;
            }
        }

        return pantsu + "</div>";
    }

    /**
     * Builds and returns the weapon section of the equipment part of the inventory.
     * @returns {string} The weapon section of the equipment part of the inventory as a HTML string.
     */
    function buildWeapon() {
        var hasWeapon = global.kobold.inventory.equipment.isWearing(SlotEnum.WEAPON);
        if (!hasWeapon) {
            return `<div class="text nomargin">You are unarmed.</div>`;
        } else {
            return `<div class="text nomargin">You are wielding a ${buildItemWithTooltip(global.kobold.inventory.equipment.weapon)}.</div>`;
        }
    }

    /**
     * Builds a comment based on how well the kobold is clothed.
     * @returns {string} The comment, as a HTML string.
     */
    function buildComment() {
        if (global.kobold.inventory.equipment.completelyNaked()) {
            return `<div class="text nomargin"><span class="warningtext">You aren't wearing anything at all.</span></div>`;
        }

        if (global.kobold.inventory.equipment.importantNaked()) {
            var shoes = global.kobold.inventory.equipment.shoes;
            var helmet = global.kobold.inventory.equipment.helmet;
            if (shoes && helmet) {
                return `<div class="text nomargin"><span class="sneakytext">Shoes and a ${helmet.name()} do little to hide your shame.</span></div>`;
            }
            if (helmet) {
                return `<div class="text nomargin"><span class="sneakytext">Your ${helmet.name()} offers little solace in face of your nudity.</span></div>`;
            }
            if (shoes) {
                return `<div class="text nomargin"><span class="sneakytext">At least you've got shoes, right?</span></div>`;
            }
        }

        return "";
    }

    /**
     * Builds and returns the equipment section of the inventory.
     * @returns {string} The equipment section of the inventory, as HTML string.
     */
    function buildEquipment() {
        var equipment = buildSubtitle("Equipment");
        equipment += buildBra();
        equipment += buildHelmet();
        equipment += buildPants();
        equipment += buildShirt();
        equipment += buildShoes();
        equipment += buildUnderwear();
        equipment += buildWeapon();
        equipment += buildComment();
        return equipment;
    }

    /**
     * Sorts the passed array of ItemAmounts by their uncapitalized names.
     * @param {array} itemAmounts The array of ItemAmounts to sort.
     * @returns {array} The sorted array.
     */
    function sortItemArray(itemAmounts) {
        itemAmounts.sort(function (a, b) {
            var aName = a.item.name();
            var bName = b.item.name();
            if (aName < bName) {
                return -1;
            }
            if (aName > bName) {
                return 1;
            }
            return 0;
        });
        return itemAmounts;
    }

    /**
     * Builds and returns the number section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The number section of an item row.
     */
    function buildItemRowNumber(itemAmount) {
        return `<div class="itemrownumber">${itemAmount.amount}</div>`;
    }

    /**
     * Builds and returns the X section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The X section of an item row.
     */
    function buildItemRowX(itemAmount) {
        return `<div class="itemrowx">x</div>`;
    }

    /**
     * Builds and returns the name section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The name section of an item row.
     */
    function buildItemRowName(itemAmount) {
        return `<div class="itemrowname">${buildItemWithTooltip(itemAmount.item, true)}</div>`;
    }

    /**
     * Builds and returns the value section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The value section of an item row.
     */
    function buildItemRowValue(itemAmount) {
        return `<div class="itemrowvalue">${Math.floor(itemAmount.item.price() * global.kobold.inventory.sellMod)} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Builds and returns the sell section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The sell section of an item row.
     */
    function buildItemRowSell(itemAmount) {
        var price = itemAmount.item.price();
        var sellZone = global.kobold.inventory.sellZone;
        var priceRight = (price > 0);

        var toReturn = `<div class="itemrowsellcontainer">`;
        if (priceRight && sellZone) {
            toReturn += `<button class="button ${sellButtonClass}">Sell</button>`;
        }
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the use section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The use section of an item row.
     */
    function buildItemRowUse(itemAmount) {
        var type = itemAmount.item.type;
        var safeZone = global.kobold.inventory.safeZone;
        var rightType = (type == ItemTypeEnum.CONSUMABLE);

        var toReturn = `<div class="itemrowusecontainer">`;
        if (rightType && safeZone) {
            toReturn += `<button class="button ${useButtonClass}">Use</button>`;
        }
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the innards of a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The innards of a row for a single item.
     */
    function buildItemRowInnards(itemAmount) {
        var innards = buildItemRowNumber(itemAmount);
        innards += buildItemRowX(itemAmount);
        innards += buildItemRowName(itemAmount);
        innards += buildItemRowValue(itemAmount);
        innards += buildItemRowSell(itemAmount);
        innards += buildItemRowUse(itemAmount);
        return innards;
    }

    /**
     * Builds and returns a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @argument {int} i The 0-based row number of the item.
     * @returns {string} A row for a single item.
     */
    function buildItemRow(itemAmount, i) {
        var id = itemAmount.item.id;
        var rowClass = "itemroweven";
        if (i % 2) {
            rowClass = "itemrowodd";
        }
        var row = `<div id="${id}" class="itemrow ${rowClass}">`;
        row += buildItemRowInnards(itemAmount);
        return row + `</div>`;
    }

    /**
     * Builds and returns the bag section of the inventory.
     * @returns {string} The bag section of the inventory, as HTML string.
     */
    function buildBag() {
        var items = global.kobold.inventory.getItemList();
        var sortedItems = sortItemArray(items);

        var bag = buildSubtitle("Backpack");
        var coins = global.kobold.inventory.coins;
        var safeZone = global.kobold.inventory.safeZone;
        var sellZone = global.kobold.inventory.sellZone;
        bag += `<div class="text">You have <span id="${coinSpanId}">${coins}</span> coin${coins !== 1 ? "s" : ""}.`;

        if (sortedItems.length > 0) {
            if (safeZone) {
                bag += ` You are in a safe zone. You may use items from the list below.`;
            } else {
                bag += ` Find a safe zone to interact with items.`;
            }
            if (sellZone) {
                bag += ` You are in a sell zone. You may sell items from the list below.`;
            }
        } else {
            if (safeZone) {
                bag += ` You are in a safe zone, but have no items to use.`;
            }
            if (sellZone) {
                bag += ` You are in a sell zone, but have no items to sell.`;
            }
        }

        bag += `</div>`;

        if (sortedItems.length < 1) {
            bag += `<div class="text">You have no items.</div>`;
        } else {
            for (var i = 0; i < sortedItems.length; i++) {
                bag += buildItemRow(sortedItems[i], i);
            }
        }
        return bag;
    }

    /**
     * Builds and returns the inventory.
     * @returns {string} The inventory, as a HTML string.
     */
    function buildInventory() {
        return buildEquipment() + buildBag();
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = gen.buildTabHeader(title);
        tab += buildInventory();
        tab += gen.buildTabFooter();
        return tab;
    }

    /**
     * Updates the kobold's coincount.
     */
    function updateCoins() {
        var coins = global.kobold.inventory.coins;
        $("#" + coinSpanId).html(coins);
    }

    /**
     * Disables the row's buttons and then hides the row.
     * @param {DOM element} row The row to hide.
     */
    function hideRow(row) {
        row.find("button").attr("disabled", true);
        row.hide(400);
    }

    /**
     * Updates the amount of the row.
     * @param {DOM element} row The row to update.
     * @param {int} newAmount The new amount of items.
     */
    function updateAmount(row, newAmount) {
        row.children(".itemrownumber").html(newAmount);
    }

    /**
     * Updates the amount of an item and removes the item if necessary.
     * @param {object} item The item of which to update the row.
     */
    function itemUpdate(item) {
        var row = $("#" + item.id);
        var amount;
        if (item.type === ItemTypeEnum.CONSUMABLE) {
            amount = global.kobold.inventory.bag.hasConsumable(item);
        } else {
            amount = global.kobold.inventory.bag.hasMisc(item);
        }
        updateAmount(row, amount);
        if (amount === 0) {
            hideRow(row);
        }
    }


    /**
     * Sells an item with the passed id.
     * @param {string | int} id The id of the item.
     */
    function sellItem(id) {
        var item = global.kobold.inventory.constructItem(id);
        global.kobold.inventory.coins += Math.floor(item.price() * global.kobold.inventory.sellMod);
        if (item.type === ItemTypeEnum.CONSUMABLE) {
            global.kobold.inventory.bag.removeConsumable(item);
        } else {
            global.kobold.inventory.bag.removeMisc(item);
        }
        itemUpdate(item);
        updateCoins();
        if (global.kobold.inventory.sellListener) {
            global.kobold.inventory.sellListener(item);
        }
    }

    /**
     * Uses an item with the passed id.
     * @param {string | int} id The id of the item.
     */
    function useItem(id) {
        global.kobold.history.consumable.use(id);
        var item = global.kobold.inventory.constructItem(id);
        item.consume();
        global.kobold.inventory.bag.removeConsumable(item);
        itemUpdate(item);
    }

    /**
     * Registers the listener for sell buttons.
     */
    function registerSellButtonListener() {
        $("." + sellButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var id = row.attr("id");
            sellItem(id);
        });
    }

    /**
     * Registers the listener for use buttons.
     */
    function registerUseButtonListener() {
        $("." + useButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var id = row.attr("id");
            useItem(id);
        });
    }

    /*
     * Registers listeners for sell and use buttons.
     */
    function registerButtonListeners() {
        registerSellButtonListener();
        registerUseButtonListener();
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the tab button start blinking if it is not already blinking.
     */
    this.startBlinking = function () {
        if (blinking) {
            return;
        }
        blinking = true;
        $("#" + id).addClass(blinkClass);
    };

    /**
     * Makes the tab button stop blinking if it is.
     */
    this.stopBlinking = function () {
        if (!blinking) {
            return;
        }
        blinking = false;
        $("#" + id).removeClass(blinkClass);
    };

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerButtonListeners();
    };
}
